@@ -20,6 +20,7 @@
 #include "folderstatusdelegate.h"
 #include "theme.h"
 
+#include <QDir>
 #include <QFileIconProvider>
 #include <QVarLengthArray>
 
@@ -41,34 +42,19 @@ namespace {
     const char propertyPermissionMap[] = "oc_permissionMap";
 }
 
-static QString removeTrailingSlash(const QString &s)
-{
-    if (s.endsWith('/')) {
-        return s.left(s.size() - 1);
-    }
-    return s;
-}
-
 FolderStatusModel::FolderStatusModel(QObject *parent)
     : QAbstractItemModel(parent)
     , _accountState(nullptr)
     , _dirty(false)
 {
+    connect(this, &FolderStatusModel::rowsInserted, this, &FolderStatusModel::dirtyChanged);
 }
 
 FolderStatusModel::~FolderStatusModel()
 {
 }
 
-static bool sortByFolderHeader(const FolderStatusModel::SubFolderInfo &lhs, const FolderStatusModel::SubFolderInfo &rhs)
-{
-    return QString::compare(lhs._folder->shortGuiRemotePathOrAppName(),
-               rhs._folder->shortGuiRemotePathOrAppName(),
-               Qt::CaseInsensitive)
-        < 0;
-}
-
-void FolderStatusModel::setAccountState(const AccountState *accountState)
+void FolderStatusModel::setAccountState(const AccountStatePtr &accountState)
 {
     beginResetModel();
     _dirty = false;
@@ -80,34 +66,34 @@ void FolderStatusModel::setAccountState(const AccountState *accountState)
     connect(FolderMan::instance(), &FolderMan::scheduleQueueChanged,
         this, &FolderStatusModel::slotFolderScheduleQueueChanged, Qt::UniqueConnection);
 
-    for (const auto &f : FolderMan::instance()->map()) {
+    for (const auto &f : FolderMan::instance()->folders()) {
         if (!accountState)
             break;
         if (f->accountState() != accountState)
             continue;
         SubFolderInfo info;
-        info._name = f->alias();
-        info._path = "/";
+        // the name is not actually used with the root element
+        info._name = QLatin1Char('/');
+        info._path = QLatin1Char('/');
         info._folder = f;
         info._checked = Qt::PartiallyChecked;
         _folders << info;
 
-        connect(f, &Folder::progressInfo, this, [f, this](const ProgressInfo &info) {
-            slotSetProgress(info, f);
+        connect(ProgressDispatcher::instance(), &ProgressDispatcher::progressInfo, this, [f, this](Folder *folder, const ProgressInfo &progress) {
+            if (folder == f) {
+                slotSetProgress(progress, f);
+            }
         });
 
         connect(f, &Folder::newBigFolderDiscovered, this, &FolderStatusModel::slotNewBigFolder, Qt::UniqueConnection);
     }
 
-    // Sort by header text
-    std::sort(_folders.begin(), _folders.end(), sortByFolderHeader);
-
     // Set the root _pathIdx after the sorting
     for (int i = 0; i < _folders.size(); ++i) {
         _folders[i]._pathIdx << i;
     }
 
-    endResetModel();
+    Q_EMIT endResetModel();
     emit dirtyChanged();
 }
 
@@ -115,7 +101,7 @@ void FolderStatusModel::setAccountState(const AccountState *accountState)
 Qt::ItemFlags FolderStatusModel::flags(const QModelIndex &index) const
 {
     if (!_accountState) {
-        return nullptr;
+        return Qt::NoItemFlags;
     }
 
     // Always enable the item. If it isn't enabled, it cannot be in the selection model, so all
@@ -150,24 +136,41 @@ QVariant FolderStatusModel::data(const QModelIndex &index, int role) const
     if (role == Qt::EditRole)
         return QVariant();
 
-    switch (classify(index)) {
+    const Columns column = static_cast<Columns>(index.column());
+    const auto itemType = classify(index);
+    if (column == Columns::ItemType) {
+        return itemType;
+    }
+    switch (itemType) {
     case AddButton: {
-        if (role == FolderStatusDelegate::AddButton) {
-            return QVariant(true);
-        } else if (role == Qt::ToolTipRole) {
+        if (role == Qt::ToolTipRole) {
             if (!_accountState->isConnected()) {
                 return tr("You need to be connected to add a folder");
             }
-            return tr("Click this button to add a folder to synchronize.");
+            return _accountState->supportsSpaces() ? tr("Click this button to add a space.") : tr("Click this button to add a folder to synchronize.");
         }
         return QVariant();
     }
     case SubFolder: {
         const auto &x = static_cast<SubFolderInfo *>(index.internalPointer())->_subs.at(index.row());
+
         switch (role) {
         case Qt::DisplayRole:
-            //: Example text: "File.txt (23KB)"
-            return x._size < 0 ? x._name : tr("%1 (%2)").arg(x._name, Utility::octetsToString(x._size));
+            switch (column) {
+            case Columns::FolderPathRole: {
+                auto f = x._folder;
+                if (!f)
+                    return QVariant();
+                return QVariant(f->path() + x._path);
+            }
+            case Columns::IsReady:
+                return x._folder->isReady();
+            case Columns::HeaderRole:
+                //: Example text: "File.txt (23KB)"
+                return x._size < 0 ? x._name : tr("%1 (%2)").arg(x._name, Utility::octetsToString(x._size));
+            default:
+                return {};
+            }
         case Qt::ToolTipRole:
             return QString(QLatin1String("<qt>") + Utility::escape(x._size < 0 ? x._name : tr("%1 (%2)").arg(x._name, Utility::octetsToString(x._size))) + QLatin1String("</qt>"));
         case Qt::CheckStateRole:
@@ -179,16 +182,6 @@ QVariant FolderStatusModel::data(const QModelIndex &index, int role) const
                 return QColor(Qt::red);
             }
             break;
-        case FolderStatusDelegate::FolderPathRole: {
-            auto f = x._folder;
-            if (!f)
-                return QVariant();
-            return QVariant(f->path() + x._path);
-        }
-        case FolderStatusDelegate::IsReady: {
-            auto f = x._folder;
-            return f->isReady();
-        }
         }
     }
         return QVariant();
@@ -198,7 +191,7 @@ QVariant FolderStatusModel::data(const QModelIndex &index, int role) const
         case Qt::DisplayRole:
             if (x->_hasError) {
                 return QVariant(tr("Error while loading the list of folders from the server.")
-                    + QString("\n") + x->_lastErrorString);
+                    + QStringLiteral("\n") + x->_lastErrorString);
             } else {
                 return tr("Fetching folder list from server...");
             }
@@ -220,73 +213,83 @@ QVariant FolderStatusModel::data(const QModelIndex &index, int role) const
     const bool accountConnected = _accountState->isConnected();
 
     switch (role) {
-    case FolderStatusDelegate::FolderPathRole:
-        return f->shortGuiLocalPath();
-    case FolderStatusDelegate::FolderSecondPathRole:
-        return f->remotePath();
-    case FolderStatusDelegate::FolderConflictMsg:
-        return (f->syncResult().hasUnresolvedConflicts())
-            ? QStringList(tr("There are unresolved conflicts. Click for details."))
-            : QStringList();
-    case FolderStatusDelegate::FolderErrorMsg: {
-        auto errors = f->syncResult().errorStrings();
-        const auto legacyError = FolderMan::instance()->unsupportedConfiguration(f->path());
-        if (!legacyError) {
-            // the error message might contain new lines, the delegate only expect multiple single line values
-            errors.append(legacyError.error().split(QLatin1Char('\n')));
+    case Qt::DisplayRole:
+        switch (column) {
+        case Columns::FolderPathRole:
+            return f->shortGuiLocalPath();
+        case Columns::FolderSecondPathRole:
+            return f->remotePath();
+        case Columns::FolderConflictMsg:
+            return (f->syncResult().hasUnresolvedConflicts())
+                ? QStringList(tr("There are unresolved conflicts. Click for details."))
+                : QStringList();
+        case Columns::FolderErrorMsg: {
+            auto errors = f->syncResult().errorStrings();
+            const auto legacyError = FolderMan::instance()->unsupportedConfiguration(f->path());
+            if (!legacyError) {
+                // the error message might contain new lines, the delegate only expect multiple single line values
+                errors.append(legacyError.error().split(QLatin1Char('\n')));
+            }
+            return errors;
         }
-        return errors;
-    }
-    case FolderStatusDelegate::FolderInfoMsg:
-        return f->isReady() && f->virtualFilesEnabled() && f->vfs().mode() != Vfs::Mode::WindowsCfApi
-            ? QStringList(tr("Virtual file support is enabled."))
-            : QStringList();
-    case FolderStatusDelegate::SyncRunning:
-        return f->syncResult().status() == SyncResult::SyncRunning;
-    case FolderStatusDelegate::HeaderRole:
-        return f->shortGuiRemotePathOrAppName();
-    case FolderStatusDelegate::FolderAliasRole:
-        return f->alias();
-    case FolderStatusDelegate::FolderSyncPaused:
-        return f->syncPaused();
-    case FolderStatusDelegate::FolderAccountConnected:
-        return accountConnected;
+        case Columns::FolderInfoMsg:
+            return f->isReady() && f->virtualFilesEnabled() && f->vfs().mode() != Vfs::Mode::WindowsCfApi
+                ? QStringList(tr("Virtual file support is enabled."))
+                : QStringList();
+        case Columns::SyncRunning:
+            return f->syncResult().status() == SyncResult::SyncRunning;
+        case Columns::HeaderRole:
+            return f->displayName();
+        case Columns::FolderSyncPaused:
+            return f->syncPaused();
+        case Columns::FolderAccountConnected:
+            return accountConnected;
+        case Columns::FolderStatusIconRole:
+            if (accountConnected) {
+                auto theme = Theme::instance();
+                if (f->syncPaused()) {
+                    return theme->folderDisabledIcon();
+                } else {
+                    return theme->syncStateIcon(f->syncResult().status());
+                }
+            } else {
+                return Theme::instance()->folderOfflineIcon();
+            }
+        case Columns::SyncProgressItemString:
+            return progress._progressString;
+        case Columns::WarningCount:
+            return progress._warningCount;
+        case Columns::SyncProgressOverallPercent:
+            return progress._overallPercent;
+        case Columns::SyncProgressOverallString:
+            return progress._overallSyncString;
+        case Columns::FolderSyncText:
+            return tr("Local folder: %1").arg(f->shortGuiLocalPath());
+        case Columns::IsReady:
+            return f->isReady();
+        case Columns::IsUsingSpaces:
+            return _accountState->supportsSpaces();
+        case Columns::IsDeployed:
+            return f->isDeployed();
+        case Columns::Priority:
+            return f->priority() + 1; // add one to have a higher prio than the hacked add button
+        case Columns::ItemType: // handled before
+            [[fallthrough]];
+        case Columns::ColumnCount:
+            Q_UNREACHABLE();
+            break;
+        }
+        break;
     case Qt::ToolTipRole: {
-        QString toolTip;
         if (!progress.isNull()) {
             return progress._progressString;
         }
-        if (accountConnected)
-            toolTip = Theme::instance()->statusHeaderText(f->syncResult().status());
-        else
-            toolTip = tr("Signed out");
-        toolTip += "\n";
-        toolTip += folderInfo._folder->path();
-        return toolTip;
-    }
-    case FolderStatusDelegate::FolderStatusIconRole:
         if (accountConnected) {
-            auto theme = Theme::instance();
-            if (f->syncPaused()) {
-                return theme->folderDisabledIcon();
-            } else {
-                return theme->syncStateIcon(f->syncResult().status());
-            }
+            return tr("%1\n%2").arg(Theme::instance()->statusHeaderText(f->syncResult().status()), QDir::toNativeSeparators(folderInfo._folder->path()));
         } else {
-            return Theme::instance()->folderOfflineIcon();
+            return tr("Signed out\n%1").arg(QDir::toNativeSeparators(folderInfo._folder->path()));
         }
-    case FolderStatusDelegate::SyncProgressItemString:
-        return progress._progressString;
-    case FolderStatusDelegate::WarningCount:
-        return progress._warningCount;
-    case FolderStatusDelegate::SyncProgressOverallPercent:
-        return progress._overallPercent;
-    case FolderStatusDelegate::SyncProgressOverallString:
-        return progress._overallSyncString;
-    case FolderStatusDelegate::FolderSyncText:
-        return tr("Local folder: %1").arg(f->shortGuiLocalPath());
-    case FolderStatusDelegate::IsReady:
-        return f->isReady();
+    }
     }
     return QVariant();
 }
@@ -357,7 +360,7 @@ bool FolderStatusModel::setData(const QModelIndex &index, const QVariant &value,
         }
         _dirty = true;
         emit dirtyChanged();
-        emit dataChanged(index, index, QVector<int>() << role);
+        emit dataChanged(index, index, { role });
         return true;
     }
     return QAbstractItemModel::setData(index, value, role);
@@ -366,7 +369,7 @@ bool FolderStatusModel::setData(const QModelIndex &index, const QVariant &value,
 
 int FolderStatusModel::columnCount(const QModelIndex &) const
 {
-    return 1;
+    return static_cast<int>(Columns::ColumnCount);
 }
 
 int FolderStatusModel::rowCount(const QModelIndex &parent) const
@@ -428,7 +431,7 @@ QModelIndex FolderStatusModel::indexForPath(Folder *f, const QString &path) cons
         return QModelIndex();
     }
 
-    int slashPos = path.lastIndexOf('/');
+    int slashPos = path.lastIndexOf(QLatin1Char('/'));
     if (slashPos == -1) {
         // first level folder
         for (int i = 0; i < _folders.size(); ++i) {
@@ -575,7 +578,7 @@ bool FolderStatusModel::canFetchMore(const QModelIndex &parent) const
 void FolderStatusModel::fetchMore(const QModelIndex &parent)
 {
     {
-        const auto isReady = data(parent, FolderStatusDelegate::IsReady);
+        const auto isReady = parent.siblingAtColumn(static_cast<int>(Columns::IsReady)).data();
 
         Q_ASSERT(isReady.isValid());
 
@@ -593,17 +596,15 @@ void FolderStatusModel::fetchMore(const QModelIndex &parent)
     if (info->_path != QLatin1String("/")) {
         path += info->_path;
     }
-    LsColJob *job = new LsColJob(_accountState->account(), path, this);
+    PropfindJob *job = new PropfindJob(_accountState->account(), info->_folder->webDavUrl(), path, PropfindJob::Depth::One, this);
     info->_fetchingJob = job;
-    job->setProperties(QList<QByteArray>() << "resourcetype"
-                                           << "http://owncloud.org/ns:size"
-                                           << "http://owncloud.org/ns:permissions");
-    job->setTimeout(60 * 1000);
-    connect(job, &LsColJob::directoryListingSubfolders,
+    job->setProperties({ QByteArrayLiteral("resourcetype"), QByteArrayLiteral("http://owncloud.org/ns:size"), QByteArrayLiteral("http://owncloud.org/ns:permissions") });
+    job->setTimeout(60s);
+    connect(job, &PropfindJob::directoryListingSubfolders,
         this, &FolderStatusModel::slotUpdateDirectories);
-    connect(job, &LsColJob::finishedWithError,
+    connect(job, &PropfindJob::finishedWithError,
         this, &FolderStatusModel::slotLscolFinishedWithError);
-    connect(job, &LsColJob::directoryListingIterated,
+    connect(job, &PropfindJob::directoryListingIterated,
         this, &FolderStatusModel::slotGatherPermissions);
 
     job->start();
@@ -613,7 +614,7 @@ void FolderStatusModel::fetchMore(const QModelIndex &parent)
 
     // Show 'fetching data...' hint after a while.
     _fetchingItems[persistentIndex].start();
-    QTimer::singleShot(1000, this, &FolderStatusModel::slotShowFetchProgress);
+    QTimer::singleShot(1s, this, &FolderStatusModel::slotShowFetchProgress);
 }
 
 void FolderStatusModel::resetAndFetch(const QModelIndex &parent)
@@ -625,7 +626,7 @@ void FolderStatusModel::resetAndFetch(const QModelIndex &parent)
 
 void FolderStatusModel::slotGatherPermissions(const QString &href, const QMap<QString, QString> &map)
 {
-    auto it = map.find("permissions");
+    auto it = map.find(QStringLiteral("permissions"));
     if (it == map.end())
         return;
 
@@ -639,7 +640,7 @@ void FolderStatusModel::slotGatherPermissions(const QString &href, const QMap<QS
 
 void FolderStatusModel::slotUpdateDirectories(const QStringList &list)
 {
-    auto job = qobject_cast<LsColJob *>(sender());
+    auto job = qobject_cast<PropfindJob *>(sender());
     OC_ASSERT(job);
     QModelIndex idx = qvariant_cast<QPersistentModelIndex>(job->property(propertyParentIndexC));
     auto parentInfo = infoForIndex(idx);
@@ -665,8 +666,8 @@ void FolderStatusModel::slotUpdateDirectories(const QStringList &list)
 
     QUrl url = parentInfo->_folder->remoteUrl();
     QString pathToRemove = url.path();
-    if (!pathToRemove.endsWith('/'))
-        pathToRemove += '/';
+    if (!pathToRemove.endsWith(QLatin1Char('/')))
+        pathToRemove += QLatin1Char('/');
 
     QStringList selectiveSyncBlackList;
     bool ok1 = true;
@@ -709,9 +710,9 @@ void FolderStatusModel::slotUpdateDirectories(const QStringList &list)
         newInfo._pathIdx = parentInfo->_pathIdx;
         newInfo._pathIdx << newSubs.size();
         newInfo._size = job->sizes().value(path);
-        newInfo._isExternal = permissionMap.value(removeTrailingSlash(path)).toString().contains("M");
+        newInfo._isExternal = permissionMap.value(Utility::stripTrailingSlash(path)).toString().contains(QLatin1String("M"));
         newInfo._path = relativePath;
-        newInfo._name = removeTrailingSlash(relativePath).split('/').last();
+        newInfo._name = Utility::stripTrailingSlash(relativePath).split(QLatin1Char('/')).last();
 
         if (relativePath.isEmpty())
             continue;
@@ -750,13 +751,13 @@ void FolderStatusModel::slotUpdateDirectories(const QStringList &list)
     }
 
     if (!newSubs.isEmpty()) {
-        beginInsertRows(idx, 0, newSubs.size() - 1);
+        Q_EMIT beginInsertRows(idx, 0, newSubs.size() - 1);
         parentInfo->_subs = std::move(newSubs);
-        endInsertRows();
+        Q_EMIT endInsertRows();
     }
 
     for (auto it = undecidedIndexes.begin(); it != undecidedIndexes.end(); ++it) {
-        suggestExpand(index(*it, 0, idx));
+        Q_EMIT suggestExpand(index(*it, 0, idx));
     }
     /* Try to remove the the undecided lists the items that are not on the server. */
     auto it = std::remove_if(selectiveSyncUndecidedList.begin(), selectiveSyncUndecidedList.end(),
@@ -771,7 +772,7 @@ void FolderStatusModel::slotUpdateDirectories(const QStringList &list)
 
 void FolderStatusModel::slotLscolFinishedWithError(QNetworkReply *r)
 {
-    auto job = qobject_cast<LsColJob *>(sender());
+    auto job = qobject_cast<PropfindJob *>(sender());
     OC_ASSERT(job);
     QModelIndex idx = qvariant_cast<QPersistentModelIndex>(job->property(propertyParentIndexC));
     if (!idx.isValid()) {
@@ -905,38 +906,36 @@ void FolderStatusModel::slotSetProgress(const ProgressInfo &progress, Folder *f)
 
     auto *pi = &folder._progress;
 
-    const QVector<int> roles = { FolderStatusDelegate::SyncProgressItemString, FolderStatusDelegate::WarningCount, Qt::ToolTipRole };
+    if (progress.status() == ProgressInfo::Done && !progress._lastCompletedItem.isEmpty()
+        && Progress::isWarningKind(progress._lastCompletedItem._status)) {
+        pi->_warningCount++;
+    }
+    // depending on the use of virtual files or small files this slot might be called very often.
+    // throttle the model updates to prevent an needlessly high cpu usage used on ui updates.
+    if (folder._lastProgressUpdateStatus != progress.status() || (std::chrono::steady_clock::now() - folder._lastProgressUpdated > progressUpdateTimeOutC)) {
+        folder._lastProgressUpdateStatus = progress.status();
 
-    switch (progress.status()) {
-    case ProgressInfo::None:
-        Q_UNREACHABLE();
-    case ProgressInfo::Discovery:
-        if (!progress._currentDiscoveredRemoteFolder.isEmpty()) {
-            pi->_overallSyncString = tr("Checking for changes in remote '%1'").arg(progress._currentDiscoveredRemoteFolder);
-            emit dataChanged(index(folderIndex), index(folderIndex), roles);
-        } else if (!progress._currentDiscoveredLocalFolder.isEmpty()) {
-            pi->_overallSyncString = tr("Checking for changes in local '%1'").arg(progress._currentDiscoveredLocalFolder);
-            emit dataChanged(index(folderIndex), index(folderIndex), roles);
-        }
-        break;
-    case ProgressInfo::Reconcile:
-        pi->_overallSyncString = tr("Reconciling changes");
-        emit dataChanged(index(folderIndex), index(folderIndex), roles);
-        break;
-    case ProgressInfo::Propagation:
-        Q_FALLTHROUGH();
-    case ProgressInfo::Done:
-        if (!progress._lastCompletedItem.isEmpty()
-            && Progress::isWarningKind(progress._lastCompletedItem._status)) {
-            pi->_warningCount++;
-        }
-
-        // progress updates are expensive, throtle them
-        if (std::chrono::steady_clock::now() - folder._lastProgressUpdated > progressUpdateTimeOutC) {
+        switch (progress.status()) {
+        case ProgressInfo::None:
+            Q_UNREACHABLE();
+            break;
+        case ProgressInfo::Discovery:
+            if (!progress._currentDiscoveredRemoteFolder.isEmpty()) {
+                pi->_overallSyncString = tr("Checking for changes in remote '%1'").arg(progress._currentDiscoveredRemoteFolder);
+            } else if (!progress._currentDiscoveredLocalFolder.isEmpty()) {
+                pi->_overallSyncString = tr("Checking for changes in local '%1'").arg(progress._currentDiscoveredLocalFolder);
+            }
+            break;
+        case ProgressInfo::Reconcile:
+            pi->_overallSyncString = tr("Reconciling changes");
+            break;
+        case ProgressInfo::Propagation:
+            Q_FALLTHROUGH();
+        case ProgressInfo::Done:
             computeProgress(progress, pi);
-            folder._lastProgressUpdated = std::chrono::steady_clock::now();
-            emit dataChanged(index(folderIndex), index(folderIndex), roles);
         }
+        emit dataChanged(index(folderIndex), index(folderIndex));
+        folder._lastProgressUpdated = std::chrono::steady_clock::now();
     }
 }
 
@@ -963,13 +962,7 @@ void FolderStatusModel::computeProgress(const ProgressInfo &progress, SubFolderI
             estimatedUpBw += progress.fileProgress(citm._item).estimatedBandwidth;
         }
         auto fileName = QFileInfo(citm._item._file).fileName();
-        if (allFilenames.length() > 0) {
-            //: Build a list of file names
-            allFilenames.append(tr(", '%1'").arg(fileName));
-        } else {
-            //: Argument is a file name
-            allFilenames.append(tr("'%1'").arg(fileName));
-        }
+        allFilenames.append(tr("'%1'").arg(fileName));
     }
     if (curItemProgress == -1) {
         curItemProgress = curItem._size;
@@ -990,7 +983,7 @@ void FolderStatusModel::computeProgress(const ProgressInfo &progress, SubFolderI
                     Utility::octetsToString(estimatedBw) );
             */
             //: Example text: "Syncing 'foo.txt', 'bar.txt'"
-            fileProgressString = tr("Syncing %1").arg(allFilenames.join(QChar()));
+            fileProgressString = tr("Syncing %1").arg(allFilenames.join(QStringLiteral(", ")));
             if (estimatedDownBw > 0) {
                 fileProgressString.append(tr(", "));
 // ifdefs: https://github.com/owncloud/client/issues/3095#issuecomment-128409294
@@ -1094,21 +1087,17 @@ void FolderStatusModel::slotFolderSyncStateChange(Folder *f)
     } else if (state == SyncResult::NotYetStarted) {
         FolderMan *folderMan = FolderMan::instance();
         int pos = folderMan->scheduleQueue().indexOf(f);
-        for (auto other : folderMan->map()) {
+        for (auto other : folderMan->folders()) {
             if (other != f && other->isSyncRunning())
                 pos += 1;
         }
-        QString message;
-        if (pos <= 0) {
-            message = tr("Waiting...");
-        } else {
-            message = tr("Waiting for %n other folder(s)...", "", pos);
+        if (pos > 0) {
+            pi._overallSyncString = tr("Waiting for %n other folder(s)...", "", pos);
         }
         pi = SubFolderInfo::Progress();
-        pi._overallSyncString = message;
     } else if (state == SyncResult::SyncPrepare) {
         pi = SubFolderInfo::Progress();
-        pi._overallSyncString = tr("Preparing to sync...");
+        pi._overallSyncString = Theme::instance()->statusHeaderText(SyncResult::SyncPrepare);
     } else if (state == SyncResult::Problem || state == SyncResult::Success) {
         // Reset the progress info after a sync.
         pi = SubFolderInfo::Progress();
@@ -1129,7 +1118,7 @@ void FolderStatusModel::slotFolderSyncStateChange(Folder *f)
 void FolderStatusModel::slotFolderScheduleQueueChanged()
 {
     // Update messages on waiting folders.
-    for (auto *f : FolderMan::instance()->map()) {
+    for (auto *f : FolderMan::instance()->folders()) {
         slotFolderSyncStateChange(f);
     }
 }
@@ -1257,7 +1246,7 @@ bool FolderStatusModel::SubFolderInfo::hasLabel() const
     return _hasError || _fetchingLabel;
 }
 
-void FolderStatusModel::SubFolderInfo::resetSubs(FolderStatusModel *model, QModelIndex index)
+void FolderStatusModel::SubFolderInfo::resetSubs(FolderStatusModel *model, const QModelIndex &index)
 {
     _fetched = false;
     if (_fetchingJob) {
