@@ -14,22 +14,19 @@
 
 #include "config.h"
 
-#include "common/utility.h"
 #include "common/asserts.h"
+#include "common/utility.h"
+#include "common/version.h"
 #include "configfile.h"
 #include "logger.h"
 #include "theme.h"
-#include "version.h"
 
 #include "creds/abstractcredentials.h"
 
 #include "csync_exclude.h"
 
-#ifndef TOKEN_AUTH_ONLY
 #include <QWidget>
 #include <QHeaderView>
-#endif
-
 #include <QCoreApplication>
 #include <QDir>
 #include <QFile>
@@ -40,7 +37,8 @@
 #include <QOperatingSystemVersion>
 #include <QStandardPaths>
 
-#define DEFAULT_MAX_LOG_LINES 20000
+#include <chrono>
+using namespace std::chrono_literals;
 
 namespace OCC {
 
@@ -71,7 +69,7 @@ const QString minChunkSizeC() { return QStringLiteral("minChunkSize"); }
 const QString maxChunkSizeC() { return QStringLiteral("maxChunkSize"); }
 const QString targetChunkUploadDurationC() { return QStringLiteral("targetChunkUploadDuration"); }
 const QString automaticLogDirC() { return QStringLiteral("logToTemporaryLogDir"); }
-const QString deleteOldLogsAfterHoursC() { return QStringLiteral("temporaryLogDirDeleteOldLogsAfterHours"); }
+const QString numberOfLogsToKeepC() { return QStringLiteral("numberOfLogsToKeep"); }
 const QString showExperimentalOptionsC() { return QStringLiteral("showExperimentalOptions"); }
 const QString clientVersionC() { return QStringLiteral("clientVersion"); }
 
@@ -105,12 +103,6 @@ static chrono::milliseconds millisecondsValue(const QSettings &setting, const QS
 ConfigFile::ConfigFile()
 {
     QSettings::setDefaultFormat(QSettings::IniFormat);
-
-    const QString config = configFile();
-
-    QSettings settings(config, QSettings::IniFormat);
-    settings.beginGroup(defaultConnection());
-
     // run init only once
     static bool init = [this]() {
         setLogHttp(logHttp());
@@ -141,103 +133,98 @@ bool ConfigFile::setConfDir(const QString &value)
 
 bool ConfigFile::optionalDesktopNotifications() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(optionalDesktopNoficationsC(), true).toBool();
 }
 
 bool ConfigFile::showInExplorerNavigationPane() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(showInExplorerNavigationPaneC(), QOperatingSystemVersion::current() >= QOperatingSystemVersion::Windows10).toBool();
 }
 
 void ConfigFile::setShowInExplorerNavigationPane(bool show)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(showInExplorerNavigationPaneC(), show);
     settings.sync();
 }
 
-int ConfigFile::timeout() const
+std::chrono::seconds ConfigFile::timeout() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
-    return settings.value(timeoutC(), 300).toInt(); // default to 5 min
+    auto settings = makeQSettings();
+    const auto val = settings.value(timeoutC()).toInt(); // default to 5 min
+    return val ? std::chrono::seconds(val) : 5min;
 }
 
 qint64 ConfigFile::chunkSize() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(chunkSizeC(), 8 * 1024 * 1024).toLongLong(); // default to 8 MiB
 }
 
 qint64 ConfigFile::maxChunkSize() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(maxChunkSizeC(), 256 * 1024 * 1024).toLongLong(); // default to 256 MiB
 }
 
 qint64 ConfigFile::minChunkSize() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(minChunkSizeC(), 8 * 1024 * 1024).toLongLong(); // default to 8 MiB
 }
 
 chrono::milliseconds ConfigFile::targetChunkUploadDuration() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return millisecondsValue(settings, targetChunkUploadDurationC(), chrono::minutes(1));
 }
 
 void ConfigFile::setOptionalDesktopNotifications(bool show)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(optionalDesktopNoficationsC(), show);
     settings.sync();
 }
 
 void ConfigFile::saveGeometry(QWidget *w)
 {
-#ifndef TOKEN_AUTH_ONLY
     OC_ASSERT(!w->objectName().isNull());
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(w->objectName());
     settings.setValue(geometryC(), w->saveGeometry());
     settings.sync();
-#endif
 }
 
 void ConfigFile::restoreGeometry(QWidget *w)
 {
-#ifndef TOKEN_AUTH_ONLY
     w->restoreGeometry(getValue(geometryC(), w->objectName()).toByteArray());
-#endif
 }
 
 void ConfigFile::saveGeometryHeader(QHeaderView *header)
 {
-#ifndef TOKEN_AUTH_ONLY
     if (!header)
         return;
     OC_ASSERT(!header->objectName().isEmpty());
 
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(header->objectName());
     settings.setValue(geometryC(), header->saveState());
     settings.sync();
-#endif
 }
 
-void ConfigFile::restoreGeometryHeader(QHeaderView *header)
+bool ConfigFile::restoreGeometryHeader(QHeaderView *header)
 {
-#ifndef TOKEN_AUTH_ONLY
-    if (!header)
-        return;
-    OC_ASSERT(!header->objectName().isNull());
+    Q_ASSERT(header && !header->objectName().isNull());
 
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(header->objectName());
-    header->restoreState(settings.value(geometryC()).toByteArray());
-#endif
+    if (settings.contains(geometryC())) {
+        header->restoreState(settings.value(geometryC()).toByteArray());
+        return true;
+    }
+    return false;
 }
 
 QVariant ConfigFile::getPolicySetting(const QString &setting, const QVariant &defaultValue) const
@@ -370,6 +357,11 @@ QString ConfigFile::configFile()
     return configPath() + Theme::instance()->configFileName();
 }
 
+QSettings ConfigFile::makeQSettings()
+{
+    return { configFile(), QSettings::IniFormat };
+}
+
 bool ConfigFile::exists()
 {
     return QFileInfo::exists(configFile());
@@ -383,7 +375,7 @@ QString ConfigFile::defaultConnection() const
 void ConfigFile::storeData(const QString &group, const QString &key, const QVariant &value)
 {
     const QString con(group.isEmpty() ? defaultConnection() : group);
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
 
     settings.beginGroup(con);
     settings.setValue(key, value);
@@ -393,7 +385,7 @@ void ConfigFile::storeData(const QString &group, const QString &key, const QVari
 void ConfigFile::removeData(const QString &group, const QString &key)
 {
     const QString con(group.isEmpty() ? defaultConnection() : group);
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
 
     settings.beginGroup(con);
     settings.remove(key);
@@ -402,7 +394,7 @@ void ConfigFile::removeData(const QString &group, const QString &key)
 bool ConfigFile::dataExists(const QString &group, const QString &key) const
 {
     const QString con(group.isEmpty() ? defaultConnection() : group);
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
 
     settings.beginGroup(con);
     return settings.contains(key);
@@ -414,7 +406,7 @@ chrono::milliseconds ConfigFile::remotePollInterval(std::chrono::seconds default
     if (connection.isEmpty())
         con = defaultConnection();
 
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(con);
 
     auto defaultPollInterval { DefaultRemotePollInterval };
@@ -445,7 +437,7 @@ void ConfigFile::setRemotePollInterval(chrono::milliseconds interval, const QStr
         qCWarning(lcConfigFile) << "Remote Poll interval of " << interval.count() << " is below five seconds.";
         return;
     }
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(con);
     settings.setValue(remotePollIntervalC(), qlonglong(interval.count()));
     settings.sync();
@@ -458,7 +450,7 @@ chrono::milliseconds ConfigFile::forceSyncInterval(std::chrono::seconds remoteFr
     QString con(connection);
     if (connection.isEmpty())
         con = defaultConnection();
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(con);
 
     auto defaultInterval = chrono::hours(2);
@@ -472,7 +464,7 @@ chrono::milliseconds ConfigFile::forceSyncInterval(std::chrono::seconds remoteFr
 
 chrono::milliseconds OCC::ConfigFile::fullLocalDiscoveryInterval() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(defaultConnection());
     return millisecondsValue(settings, fullLocalDiscoveryIntervalC(), chrono::hours(1));
 }
@@ -482,7 +474,7 @@ chrono::milliseconds ConfigFile::notificationRefreshInterval(const QString &conn
     QString con(connection);
     if (connection.isEmpty())
         con = defaultConnection();
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(con);
 
     auto defaultInterval = chrono::minutes(5);
@@ -499,7 +491,7 @@ chrono::milliseconds ConfigFile::updateCheckInterval(const QString &connection)
     QString con(connection);
     if (connection.isEmpty())
         con = defaultConnection();
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(con);
 
     auto defaultInterval = chrono::hours(10);
@@ -532,7 +524,7 @@ void ConfigFile::setSkipUpdateCheck(bool skip, const QString &connection)
     if (connection.isEmpty())
         con = defaultConnection();
 
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(con);
 
     settings.setValue(skipUpdateCheckC(), QVariant(skip));
@@ -542,7 +534,7 @@ void ConfigFile::setSkipUpdateCheck(bool skip, const QString &connection)
 QString ConfigFile::updateChannel() const
 {
     QString defaultUpdateChannel = QStringLiteral("stable");
-    const QString suffix = MIRALL_VERSION_SUFFIX();
+    const QString suffix = OCC::Version::suffix();
     if (suffix.startsWith(QLatin1String("daily"))
         || suffix.startsWith(QLatin1String("nightly"))
         || suffix.startsWith(QLatin1String("alpha"))
@@ -551,25 +543,25 @@ QString ConfigFile::updateChannel() const
         defaultUpdateChannel = QStringLiteral("beta");
     }
 
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(updateChannelC(), defaultUpdateChannel).toString();
 }
 
 void ConfigFile::setUpdateChannel(const QString &channel)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(updateChannelC(), channel);
 }
 
 QString ConfigFile::uiLanguage() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(uiLanguageC(), QString()).toString();
 }
 
 void ConfigFile::setUiLanguage(const QString &uiLanguage)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(uiLanguageC(), uiLanguage);
 }
 
@@ -595,7 +587,7 @@ void ConfigFile::setProxyType(int proxyType,
     const QString &user,
     const QString &pass)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
 
     settings.setValue(proxyTypeC(), proxyType);
 
@@ -635,7 +627,7 @@ QVariant ConfigFile::getValue(const QString &param, const QString &group,
         systemSetting = systemSettings.value(param, defaultValue);
     }
 
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     if (!group.isEmpty())
         settings.beginGroup(group);
 
@@ -644,7 +636,7 @@ QVariant ConfigFile::getValue(const QString &param, const QString &group,
 
 void ConfigFile::setValue(const QString &key, const QVariant &value)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
 
     settings.setValue(key, value);
 }
@@ -759,19 +751,19 @@ void ConfigFile::setMoveToTrash(bool isChecked)
 
 bool ConfigFile::promptDeleteFiles() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(promptDeleteC(), true).toBool();
 }
 
 void ConfigFile::setPromptDeleteFiles(bool promptDeleteFiles)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(promptDeleteC(), promptDeleteFiles);
 }
 
 bool ConfigFile::monoIcons() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     bool monoDefault = false; // On Mac we want bw by default
 #ifdef Q_OS_MAC
     // OEM themes are not obliged to ship mono icons
@@ -782,59 +774,49 @@ bool ConfigFile::monoIcons() const
 
 void ConfigFile::setMonoIcons(bool useMonoIcons)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(monoIconsC(), useMonoIcons);
 }
 
 bool ConfigFile::crashReporter() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(crashReporterC(), true).toBool();
 }
 
 void ConfigFile::setCrashReporter(bool enabled)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(crashReporterC(), enabled);
 }
 
 bool ConfigFile::automaticLogDir() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(automaticLogDirC(), false).toBool();
 }
 
 void ConfigFile::setAutomaticLogDir(bool enabled)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(automaticLogDirC(), enabled);
 }
 
-Optional<chrono::hours> ConfigFile::automaticDeleteOldLogsAge() const
+int ConfigFile::automaticDeleteOldLogs() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
-    auto value = settings.value(deleteOldLogsAfterHoursC());
-    if (!value.isValid())
-        return chrono::hours(4);
-    auto hours = value.toInt();
-    if (hours <= 0)
-        return {};
-    return chrono::hours(hours);
+    auto settings = makeQSettings();
+    return settings.value(numberOfLogsToKeepC()).toInt();
 }
 
-void ConfigFile::setAutomaticDeleteOldLogsAge(Optional<chrono::hours> expireTime)
+void ConfigFile::setAutomaticDeleteOldLogs(int number)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
-    if (!expireTime) {
-        settings.setValue(deleteOldLogsAfterHoursC(), -1);
-    } else {
-        settings.setValue(deleteOldLogsAfterHoursC(), QVariant::fromValue(expireTime->count()));
-    }
+    auto settings = makeQSettings();
+    settings.setValue(numberOfLogsToKeepC(), number);
 }
 
 void ConfigFile::setLogHttp(bool b)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(logHttpC(), b);
     const QSet<QString> rule = { QStringLiteral("sync.httplogger=true") };
     if (b) {
@@ -846,31 +828,32 @@ void ConfigFile::setLogHttp(bool b)
 
 bool ConfigFile::logHttp() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(logHttpC(), false).toBool();
 }
 
 bool ConfigFile::showExperimentalOptions() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(showExperimentalOptionsC(), false).toBool();
 }
 
 QString ConfigFile::clientVersionString() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(clientVersionC(), QString()).toString();
 }
 
 void ConfigFile::setClientVersionString(const QString &version)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(clientVersionC(), version);
 }
 
-std::unique_ptr<QSettings> ConfigFile::settingsWithGroup(const QString &group, QObject *parent)
+std::unique_ptr<QSettings> ConfigFile::settingsWithGroup(const QString &group)
 {
-    std::unique_ptr<QSettings> settings(new QSettings(ConfigFile::configFile(), QSettings::IniFormat, parent));
+    // this actually works by move magic
+    std::unique_ptr<QSettings> settings(new QSettings(makeQSettings()));
     settings->beginGroup(group);
     return settings;
 }
