@@ -27,17 +27,17 @@
 #include "propagateremotedelete.h"
 #include "common/asserts.h"
 
-#include <QNetworkAccessManager>
-#include <QFileInfo>
+#include <QCoreApplication>
 #include <QDir>
+#include <QFileInfo>
+#include <QNetworkAccessManager>
+#include <QRandomGenerator>
 
-#include <cmath>
-#include <cstring>
 #include <memory>
 
 namespace OCC {
 
-QUrl PropagateUploadFileNG::chunkUrl(qint64 chunkOffset)
+QString PropagateUploadFileNG::chunkPath(qint64 chunkOffset)
 {
     QString path = QLatin1String("remote.php/dav/uploads/")
         + propagator()->account()->davUser()
@@ -46,7 +46,7 @@ QUrl PropagateUploadFileNG::chunkUrl(qint64 chunkOffset)
         // We need to do add leading 0 because the server orders the chunk alphabetically
         path += QLatin1Char('/') + QString::number(chunkOffset).rightJustified(16, QLatin1Char('0')); // 1e16 is 10 petabyte
     }
-    return Utility::concatUrlPath(propagator()->account()->url(), path);
+    return path;
 }
 
 
@@ -82,6 +82,11 @@ State machine:
         +-> MOVE +-----> moveJobFinished() +--> finalize()
  */
 
+PropagateUploadFileNG::PropagateUploadFileNG(OwncloudPropagator *propagator, const SyncFileItemPtr &item)
+    : PropagateUploadFileCommon(propagator, item)
+    , _bytesToUpload(item->_size)
+{
+}
 void PropagateUploadFileNG::doStartUpload()
 {
     const QString fileName = propagator()->fullLocalPath(_item->_file);
@@ -109,16 +114,13 @@ void PropagateUploadFileNG::doStartUploadNext()
     if (progressInfo._valid && progressInfo.isChunked() && progressInfo._modtime == _item->_modtime
             && progressInfo._size == _item->_size) {
         _transferId = progressInfo._transferid;
-        auto url = chunkUrl();
-        auto job = new LsColJob(propagator()->account(), url, this);
-        _jobs.append(job);
-        job->setProperties(QList<QByteArray>() << "resourcetype"
-                                               << "getcontentlength");
-        connect(job, &LsColJob::finishedWithoutError, this, &PropagateUploadFileNG::slotPropfindFinished);
-        connect(job, &LsColJob::finishedWithError,
+        auto job = new PropfindJob(propagator()->account(), propagator()->account()->url(), chunkPath(), PropfindJob::Depth::One, this);
+        addChildJob(job);
+        job->setProperties({ QByteArrayLiteral("resourcetype"), QByteArrayLiteral("getcontentlength") });
+        connect(job, &PropfindJob::finishedWithoutError, this, &PropagateUploadFileNG::slotPropfindFinished);
+        connect(job, &PropfindJob::finishedWithError,
             this, &PropagateUploadFileNG::slotPropfindFinishedWithError);
-        connect(job, &QObject::destroyed, this, &PropagateUploadFileCommon::slotJobDestroyed);
-        connect(job, &LsColJob::directoryListingIterated,
+        connect(job, &PropfindJob::directoryListingIterated,
             this, &PropagateUploadFileNG::slotPropfindIterate);
         job->start();
         return;
@@ -126,7 +128,7 @@ void PropagateUploadFileNG::doStartUploadNext()
         // The upload info is stale. remove the stale chunks on the server
         _transferId = progressInfo._transferid;
         // Fire and forget. Any error will be ignored.
-        (new DeleteJob(propagator()->account(), chunkUrl(), this))->start();
+        (new DeleteJob(propagator()->account(), propagator()->account()->url(), chunkPath(), this))->start();
         // startNewUpload will reset the _transferId and the UploadInfo in the db.
     }
 
@@ -135,7 +137,7 @@ void PropagateUploadFileNG::doStartUploadNext()
 
 void PropagateUploadFileNG::slotPropfindIterate(const QString &name, const QMap<QString, QString> &properties)
 {
-    if (name == chunkUrl().path()) {
+    if (name.endsWith(chunkPath())) {
         return; // skip the info about the path itself
     }
     bool ok = false;
@@ -169,8 +171,6 @@ bool PropagateUploadFileNG::markRangeAsDone(qint64 start, qint64 size)
 
 void PropagateUploadFileNG::slotPropfindFinished()
 {
-    auto job = qobject_cast<LsColJob *>(sender());
-    slotJobDestroyed(job); // remove it from the _jobs list
     propagator()->_activeJobList.removeOne(this);
 
     _currentChunkOffset = 0;
@@ -199,7 +199,7 @@ void PropagateUploadFileNG::slotPropfindFinished()
 
         // Wipe the old chunking data.
         // Fire and forget. Any error will be ignored.
-        (new DeleteJob(propagator()->account(), chunkUrl(), this))->start();
+        (new DeleteJob(propagator()->account(), propagator()->account()->url(), chunkPath(), this))->start();
 
         propagator()->_activeJobList.append(this);
         startNewUpload();
@@ -217,9 +217,9 @@ void PropagateUploadFileNG::slotPropfindFinished()
         // we should remove the later chunks. Otherwise when we do dynamic chunk sizing, we may end up
         // with corruptions if there are too many chunks, or if we abort and there are still stale chunks.
         for (auto it = _serverChunks.begin(); it != _serverChunks.end(); ++it) {
-            auto job = new DeleteJob(propagator()->account(), Utility::concatUrlPath(chunkUrl(), it->originalName), this);
-            QObject::connect(job, &DeleteJob::finishedSignal, this, &PropagateUploadFileNG::slotDeleteJobFinished);
-            _jobs.append(job);
+            auto job = new DeleteJob(propagator()->account(), propagator()->account()->url(), chunkPath() + QLatin1Char('/') + it->originalName, this);
+            addChildJob(job);
+            connect(job, &DeleteJob::finishedSignal, this, &PropagateUploadFileNG::slotDeleteJobFinished);
             job->start();
         }
         _serverChunks.clear();
@@ -231,13 +231,14 @@ void PropagateUploadFileNG::slotPropfindFinished()
 
 void PropagateUploadFileNG::slotPropfindFinishedWithError()
 {
-    auto job = qobject_cast<LsColJob *>(sender());
-    slotJobDestroyed(job); // remove it from the _jobs list
+    auto job = qobject_cast<PropfindJob *>(sender());
+    _item->_httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+    _item->_responseTimeStamp = job->responseTimestamp();
+    _item->_requestId = job->requestId();
+
     QNetworkReply::NetworkError err = job->reply()->error();
-    auto httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-    auto status = classifyError(err, httpErrorCode, &propagator()->_anotherSyncNeeded);
+    auto status = classifyError(err, _item->_httpErrorCode, &propagator()->_anotherSyncNeeded);
     if (status == SyncFileItem::FatalError) {
-        _item->_requestId = job->requestId();
         propagator()->_activeJobList.removeOne(this);
         abortWithError(status, job->errorStringParsingBody());
         return;
@@ -248,15 +249,15 @@ void PropagateUploadFileNG::slotPropfindFinishedWithError()
 void PropagateUploadFileNG::slotDeleteJobFinished()
 {
     auto job = qobject_cast<DeleteJob *>(sender());
-    OC_ASSERT(job);
-    _jobs.remove(_jobs.indexOf(job));
+    _item->_httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+    _item->_responseTimeStamp = job->responseTimestamp();
+    _item->_requestId = job->requestId();
 
     QNetworkReply::NetworkError err = job->reply()->error();
     if (err != QNetworkReply::NoError && err != QNetworkReply::ContentNotFoundError) {
         const int httpStatus = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
         SyncFileItem::Status status = classifyError(err, httpStatus);
         if (status == SyncFileItem::FatalError) {
-            _item->_requestId = job->requestId();
             abortWithError(status, job->errorString());
             return;
         } else {
@@ -268,7 +269,7 @@ void PropagateUploadFileNG::slotDeleteJobFinished()
 
     // If no more Delete jobs are running, we can continue
     bool runningDeleteJobs = false;
-    for (auto *otherJob : qAsConst(_jobs)) {
+    for (auto *otherJob : childJobs()) {
         if (qobject_cast<DeleteJob *>(otherJob))
             runningDeleteJobs = true;
     }
@@ -283,11 +284,10 @@ void PropagateUploadFileNG::slotDeleteJobFinished()
     }
 }
 
-
 void PropagateUploadFileNG::startNewUpload()
 {
     OC_ASSERT(propagator()->_activeJobList.count(this) == 1);
-    _transferId = uint(qrand()) ^ uint(_item->_modtime) ^ (uint(_item->_size) << 16) ^ qHash(_item->_file);
+    _transferId = QRandomGenerator::global()->generate();
     _sent = 0;
 
     propagator()->reportProgress(*_item, 0);
@@ -302,12 +302,11 @@ void PropagateUploadFileNG::startNewUpload()
     propagator()->_journal->commit(QStringLiteral("Upload info"));
     QMap<QByteArray, QByteArray> headers;
     headers["OC-Total-Length"] = QByteArray::number(_item->_size);
-    auto job = new MkColJob(propagator()->account(), chunkUrl(), headers, this);
+    auto job = new MkColJob(propagator()->account(), propagator()->account()->url(), chunkPath(), headers, this);
     connect(job, &MkColJob::finishedWithError,
         this, &PropagateUploadFileNG::slotMkColFinished);
     connect(job, &MkColJob::finishedWithoutError,
         this, &PropagateUploadFileNG::slotMkColFinished);
-    connect(job, &QObject::destroyed, this, &PropagateUploadFileCommon::slotJobDestroyed);
     job->start();
 }
 
@@ -315,10 +314,11 @@ void PropagateUploadFileNG::slotMkColFinished()
 {
     propagator()->_activeJobList.removeOne(this);
     auto job = qobject_cast<MkColJob *>(sender());
-    slotJobDestroyed(job); // remove it from the _jobs list
-    QNetworkReply::NetworkError err = job->reply()->error();
     _item->_httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+    _item->_responseTimeStamp = job->responseTimestamp();
+    _item->_requestId = job->requestId();
 
+    QNetworkReply::NetworkError err = job->reply()->error();
     if (err != QNetworkReply::NoError || _item->_httpErrorCode != 201) {
         _item->_requestId = job->requestId();
         SyncFileItem::Status status = classifyError(err, _item->_httpErrorCode,
@@ -335,13 +335,12 @@ void PropagateUploadFileNG::doFinalMove()
     // Still not finished all ranges.
     if (!_rangesToUpload.isEmpty())
         return;
-
-    OC_ENFORCE_X(_jobs.isEmpty(), "MOVE for upload even though jobs are still running");
+    Q_ASSERT_X(childJobs().empty(), Q_FUNC_INFO, "MOVE for upload even though jobs are still running");
 
     _finished = true;
 
     // Finish with a MOVE
-    QString destination = QDir::cleanPath(propagator()->account()->davUrl().path()
+    QString destination = QDir::cleanPath(propagator()->webDavUrl().path()
         + propagator()->fullRemotePath(_item->_file));
     auto headers = PropagateUploadFileCommon::headers();
 
@@ -356,7 +355,7 @@ void PropagateUploadFileNG::doFinalMove()
     headers[QByteArrayLiteral("OC-Total-Length")] = QByteArray::number(_bytesToUpload);
     headers[QByteArrayLiteral("OC-Total-File-Length")] = QByteArray::number(_item->_size);
 
-    const QUrl source = Utility::concatUrlPath(chunkUrl(), QStringLiteral("/.file"));
+    const QString source = chunkPath() + QStringLiteral("/.file");
 
 #ifdef Q_OS_WIN
     // Try to accuire a lock on the file and keep it until we done.
@@ -371,10 +370,9 @@ void PropagateUploadFileNG::doFinalMove()
     }
 #endif
 
-    auto job = new MoveJob(propagator()->account(), source, destination, headers, this);
-    _jobs.append(job);
+    auto job = new MoveJob(propagator()->account(), propagator()->account()->url(), source, destination, headers, this);
+    addChildJob(job);
     connect(job, &MoveJob::finishedSignal, this, &PropagateUploadFileNG::slotMoveJobFinished);
-    connect(job, &QObject::destroyed, this, &PropagateUploadFileCommon::slotJobDestroyed);
     propagator()->_activeJobList.append(this);
     adjustLastJobTimeout(job, _item->_size);
     job->start();
@@ -398,8 +396,8 @@ void PropagateUploadFileNG::startNextChunk()
     _currentChunkSize = qMin(propagator()->_chunkSize, _rangesToUpload.first().size);
 
     const QString fileName = propagator()->fullLocalPath(_item->_file);
-    auto device = std::unique_ptr<UploadDevice>(new UploadDevice(
-            fileName, _currentChunkOffset, _currentChunkSize, &propagator()->_bandwidthManager));
+    auto device = std::make_unique<UploadDevice>(fileName, _currentChunkOffset, _currentChunkSize,
+        propagator()->_bandwidthManager);
     if (!device->open(QIODevice::ReadOnly)) {
         qCWarning(lcPropagateUploadNG) << "Could not prepare upload device: " << device->errorString();
         // Soft error because this is likely caused by the user modifying his files while syncing
@@ -407,21 +405,15 @@ void PropagateUploadFileNG::startNextChunk()
         return;
     }
 
-    QMap<QByteArray, QByteArray> headers;
-    headers["OC-Chunk-Offset"] = QByteArray::number(_currentChunkOffset);
-
-    QUrl url = chunkUrl(_currentChunkOffset);
-
     // job takes ownership of device via a QScopedPointer. Job deletes itself when finishing
     auto devicePtr = device.get(); // for connections later
-    PUTFileJob *job = new PUTFileJob(propagator()->account(), url, std::move(device), headers, 0, this);
-    _jobs.append(job);
+    PUTFileJob *job = new PUTFileJob(propagator()->account(), propagator()->account()->url(), chunkPath(_currentChunkOffset), std::move(device), {}, 0, this);
+    addChildJob(job);
     connect(job, &PUTFileJob::finishedSignal, this, &PropagateUploadFileNG::slotPutFinished);
     connect(job, &PUTFileJob::uploadProgress,
         this, &PropagateUploadFileNG::slotUploadProgress);
     connect(job, &PUTFileJob::uploadProgress,
         devicePtr, &UploadDevice::slotJobUploadProgress);
-    connect(job, &QObject::destroyed, this, &PropagateUploadFileCommon::slotJobDestroyed);
     job->start();
     propagator()->_activeJobList.append(this);
 }
@@ -431,7 +423,9 @@ void PropagateUploadFileNG::slotPutFinished()
     PUTFileJob *job = qobject_cast<PUTFileJob *>(sender());
     OC_ASSERT(job);
 
-    slotJobDestroyed(job); // remove it from the _jobs list
+    _item->_httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+    _item->_responseTimeStamp = job->responseTimestamp();
+    _item->_requestId = job->requestId();
 
     propagator()->_activeJobList.removeOne(this);
 
@@ -443,8 +437,6 @@ void PropagateUploadFileNG::slotPutFinished()
     QNetworkReply::NetworkError err = job->reply()->error();
 
     if (err != QNetworkReply::NoError) {
-        _item->_httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-        _item->_requestId = job->requestId();
         commonErrorHandling(job);
         return;
     }
@@ -506,7 +498,7 @@ void PropagateUploadFileNG::slotPutFinished()
     if (FileSystem::fileChanged(fullFilePath, _item->_size, _item->_modtime)) {
         propagator()->_anotherSyncNeeded = true;
         if (!_finished) {
-            abortWithError(SyncFileItem::SoftError, tr("Local file changed during sync."));
+            abortWithError(SyncFileItem::Message, fileChangedMessage());
             return;
         }
     }
@@ -531,7 +523,6 @@ void PropagateUploadFileNG::slotMoveJobFinished()
 {
     propagator()->_activeJobList.removeOne(this);
     auto job = qobject_cast<MoveJob *>(sender());
-    slotJobDestroyed(job); // remove it from the _jobs list
     QNetworkReply::NetworkError err = job->reply()->error();
     _item->_httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
     _item->_responseTimeStamp = job->responseTimestamp();
@@ -566,7 +557,6 @@ void PropagateUploadFileNG::slotMoveJobFinished()
     }
 
     _item->_etag = getEtagFromReply(job->reply());
-    ;
     if (_item->_etag.isEmpty()) {
         qCWarning(lcPropagateUploadNG) << "Server did not return an ETAG" << _item->_file;
         abortWithError(SyncFileItem::NormalError, tr("Missing ETag from server"));
