@@ -18,14 +18,18 @@ public:
     StatusPushSpy(SyncEngine &syncEngine)
         : QSignalSpy(&syncEngine.syncFileStatusTracker(), &SyncFileStatusTracker::fileStatusChanged)
         , _syncEngine(syncEngine)
-    { }
+    {
+    }
 
-    SyncFileStatus statusOf(const QString &relativePath) const {
-        QFileInfo file(_syncEngine.localPath(), relativePath);
+    SyncFileStatus statusOf(const QString &relativePath) const
+    {
+        const QFileInfo file(_syncEngine.localPath(), relativePath);
         // Start from the end to get the latest status
         for (auto it = crbegin(); it != crend(); ++it) {
-            if (QFileInfo(it->at(0).toString()) == file)
+            const auto info = QFileInfo(it->at(0).toString());
+            if (info == file) {
                 return it->at(1).value<SyncFileStatus>();
+            }
         }
         return SyncFileStatus();
     }
@@ -49,6 +53,11 @@ public:
     }
 };
 
+//
+// IMPORTANT: these tests are only run with VFS off, because the tests assume a very controlled order of status changes.
+// When using some form of VFS, the event loop has to run in order to fulfill requests from the OS, and therefore things
+// like stopping execution in `execUntilBeforePropagation()` are very hard to do.
+//
 class TestSyncFileStatusTracker : public QObject
 {
     Q_OBJECT
@@ -58,7 +67,7 @@ class TestSyncFileStatusTracker : public QObject
         QDirIterator it(root, QDir::AllEntries | QDir::NoDotAndDotDot, QDirIterator::Subdirectories);
         while (it.hasNext()) {
             QString filePath = it.next().mid(root.size());
-            SyncFileStatus pushedStatus = statusSpy.statusOf(filePath);
+            auto pushedStatus = statusSpy.statusOf(filePath);
             if (pushedStatus != SyncFileStatus()) {
                 QCOMPARE(fakeFolder.syncEngine().syncFileStatusTracker().fileStatus(filePath), pushedStatus);
             }
@@ -68,8 +77,9 @@ class TestSyncFileStatusTracker : public QObject
 private slots:
     void parentsGetSyncStatusUploadDownload() {
         FakeFolder fakeFolder{FileInfo::A12_B12_C12_S12()};
-        fakeFolder.localModifier().appendByte("B/b1");
-        fakeFolder.remoteModifier().appendByte("C/c1");
+        fakeFolder.localModifier().appendByte(QStringLiteral("B/b1"));
+        fakeFolder.applyLocalModificationsWithoutSync();
+        fakeFolder.remoteModifier().appendByte(QStringLiteral("C/c1"));
         StatusPushSpy statusSpy(fakeFolder.syncEngine());
 
         fakeFolder.scheduleSync();
@@ -99,8 +109,9 @@ private slots:
 
     void parentsGetSyncStatusNewFileUploadDownload() {
         FakeFolder fakeFolder{FileInfo::A12_B12_C12_S12()};
-        fakeFolder.localModifier().insert("B/b0");
-        fakeFolder.remoteModifier().insert("C/c0");
+        fakeFolder.localModifier().insert(QStringLiteral("B/b0"));
+        fakeFolder.applyLocalModificationsWithoutSync();
+        fakeFolder.remoteModifier().insert(QStringLiteral("C/c0"));
         StatusPushSpy statusSpy(fakeFolder.syncEngine());
 
         fakeFolder.scheduleSync();
@@ -130,8 +141,8 @@ private slots:
 
     void parentsGetSyncStatusNewDirDownload() {
         FakeFolder fakeFolder{FileInfo::A12_B12_C12_S12()};
-        fakeFolder.remoteModifier().mkdir("D");
-        fakeFolder.remoteModifier().insert("D/d0");
+        fakeFolder.remoteModifier().mkdir(QStringLiteral("D"));
+        fakeFolder.remoteModifier().insert(QStringLiteral("D/d0"));
         StatusPushSpy statusSpy(fakeFolder.syncEngine());
 
         fakeFolder.scheduleSync();
@@ -141,25 +152,28 @@ private slots:
         QCOMPARE(statusSpy.statusOf("D"), SyncFileStatus(SyncFileStatus::StatusSync));
         QCOMPARE(statusSpy.statusOf("D/d0"), SyncFileStatus(SyncFileStatus::StatusSync));
 
-        fakeFolder.execUntilItemCompleted("D");
-        verifyThatPushMatchesPull(fakeFolder, statusSpy);
-        QCOMPARE(statusSpy.statusOf(""), SyncFileStatus(SyncFileStatus::StatusSync));
-        QCOMPARE(statusSpy.statusOf("D"), SyncFileStatus(SyncFileStatus::StatusSync));
-        QCOMPARE(statusSpy.statusOf("D/d0"), SyncFileStatus(SyncFileStatus::StatusSync));
-
-        fakeFolder.execUntilFinished();
+        statusSpy.clear();
+        fakeFolder.execUntilItemCompleted(QStringLiteral("D"));
         verifyThatPushMatchesPull(fakeFolder, statusSpy);
         QCOMPARE(statusSpy.statusOf(""), SyncFileStatus(SyncFileStatus::StatusUpToDate));
         QCOMPARE(statusSpy.statusOf("D"), SyncFileStatus(SyncFileStatus::StatusUpToDate));
         QCOMPARE(statusSpy.statusOf("D/d0"), SyncFileStatus(SyncFileStatus::StatusUpToDate));
 
+        statusSpy.clear();
+        fakeFolder.execUntilFinished();
+        verifyThatPushMatchesPull(fakeFolder, statusSpy);
+        QCOMPARE(statusSpy.statusOf(""), SyncFileStatus(SyncFileStatus::StatusUpToDate));
+        QCOMPARE(statusSpy.statusOf("D"), SyncFileStatus(SyncFileStatus::StatusNone));
+        QCOMPARE(statusSpy.statusOf("D/d0"), SyncFileStatus(SyncFileStatus::StatusNone));
+
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
     }
 
     void parentsGetSyncStatusNewDirUpload() {
         FakeFolder fakeFolder{FileInfo::A12_B12_C12_S12()};
-        fakeFolder.localModifier().mkdir("D");
-        fakeFolder.localModifier().insert("D/d0");
+        fakeFolder.localModifier().mkdir(QStringLiteral("D"));
+        fakeFolder.localModifier().insert(QStringLiteral("D/d0"));
+        fakeFolder.applyLocalModificationsWithoutSync();
         StatusPushSpy statusSpy(fakeFolder.syncEngine());
 
         fakeFolder.scheduleSync();
@@ -169,25 +183,28 @@ private slots:
         QCOMPARE(statusSpy.statusOf("D"), SyncFileStatus(SyncFileStatus::StatusSync));
         QCOMPARE(statusSpy.statusOf("D/d0"), SyncFileStatus(SyncFileStatus::StatusSync));
 
-        fakeFolder.execUntilItemCompleted("D");
-        verifyThatPushMatchesPull(fakeFolder, statusSpy);
-        QCOMPARE(statusSpy.statusOf(""), SyncFileStatus(SyncFileStatus::StatusSync));
-        QCOMPARE(statusSpy.statusOf("D"), SyncFileStatus(SyncFileStatus::StatusSync));
-        QCOMPARE(statusSpy.statusOf("D/d0"), SyncFileStatus(SyncFileStatus::StatusSync));
-
-        fakeFolder.execUntilFinished();
+        statusSpy.clear();
+        fakeFolder.execUntilItemCompleted(QStringLiteral("D"));
         verifyThatPushMatchesPull(fakeFolder, statusSpy);
         QCOMPARE(statusSpy.statusOf(""), SyncFileStatus(SyncFileStatus::StatusUpToDate));
         QCOMPARE(statusSpy.statusOf("D"), SyncFileStatus(SyncFileStatus::StatusUpToDate));
         QCOMPARE(statusSpy.statusOf("D/d0"), SyncFileStatus(SyncFileStatus::StatusUpToDate));
 
+        statusSpy.clear();
+        fakeFolder.execUntilFinished();
+        verifyThatPushMatchesPull(fakeFolder, statusSpy);
+        QCOMPARE(statusSpy.statusOf(""), SyncFileStatus(SyncFileStatus::StatusUpToDate));
+        QCOMPARE(statusSpy.statusOf("D"), SyncFileStatus(SyncFileStatus::StatusNone));
+        QCOMPARE(statusSpy.statusOf("D/d0"), SyncFileStatus(SyncFileStatus::StatusNone));
+
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
     }
 
     void parentsGetSyncStatusDeleteUpDown() {
         FakeFolder fakeFolder{FileInfo::A12_B12_C12_S12()};
-        fakeFolder.remoteModifier().remove("B/b1");
-        fakeFolder.localModifier().remove("C/c1");
+        fakeFolder.remoteModifier().remove(QStringLiteral("B/b1"));
+        fakeFolder.localModifier().remove(QStringLiteral("C/c1"));
+        fakeFolder.applyLocalModificationsWithoutSync();
         StatusPushSpy statusSpy(fakeFolder.syncEngine());
 
         fakeFolder.scheduleSync();
@@ -214,10 +231,11 @@ private slots:
 
     void warningStatusForExcludedFile() {
         FakeFolder fakeFolder{FileInfo::A12_B12_C12_S12()};
-        fakeFolder.syncEngine().excludedFiles().addManualExclude("A/a1");
-        fakeFolder.syncEngine().excludedFiles().addManualExclude("B");
-        fakeFolder.localModifier().appendByte("A/a1");
-        fakeFolder.localModifier().appendByte("B/b1");
+        fakeFolder.syncEngine().excludedFiles().addManualExclude(QStringLiteral("A/a1"));
+        fakeFolder.syncEngine().excludedFiles().addManualExclude(QStringLiteral("B"));
+        fakeFolder.localModifier().appendByte(QStringLiteral("A/a1"));
+        fakeFolder.localModifier().appendByte(QStringLiteral("B/b1"));
+        fakeFolder.applyLocalModificationsWithoutSync();
         StatusPushSpy statusSpy(fakeFolder.syncEngine());
 
         fakeFolder.scheduleSync();
@@ -264,9 +282,10 @@ private slots:
 
     void warningStatusForExcludedFile_CasePreserving() {
         FakeFolder fakeFolder{FileInfo::A12_B12_C12_S12()};
-        fakeFolder.syncEngine().excludedFiles().addManualExclude("B");
-        fakeFolder.serverErrorPaths().append("A/a1");
-        fakeFolder.localModifier().appendByte("A/a1");
+        fakeFolder.syncEngine().excludedFiles().addManualExclude(QStringLiteral("B"));
+        fakeFolder.serverErrorPaths().append(QStringLiteral("A/a1"));
+        fakeFolder.localModifier().appendByte(QStringLiteral("A/a1"));
+        fakeFolder.applyLocalModificationsWithoutSync();
 
         fakeFolder.syncOnce();
         QCOMPARE(fakeFolder.syncEngine().syncFileStatusTracker().fileStatus(""), SyncFileStatus(SyncFileStatus::StatusWarning));
@@ -282,10 +301,11 @@ private slots:
 
     void parentsGetWarningStatusForError() {
         FakeFolder fakeFolder{FileInfo::A12_B12_C12_S12()};
-        fakeFolder.serverErrorPaths().append("A/a1");
-        fakeFolder.serverErrorPaths().append("B/b0");
-        fakeFolder.localModifier().appendByte("A/a1");
-        fakeFolder.localModifier().insert("B/b0");
+        fakeFolder.serverErrorPaths().append(QStringLiteral("A/a1"));
+        fakeFolder.serverErrorPaths().append(QStringLiteral("B/b0"));
+        fakeFolder.localModifier().appendByte(QStringLiteral("A/a1"));
+        fakeFolder.localModifier().insert(QStringLiteral("B/b0"));
+        fakeFolder.applyLocalModificationsWithoutSync();
         StatusPushSpy statusSpy(fakeFolder.syncEngine());
 
         fakeFolder.scheduleSync();
@@ -331,7 +351,8 @@ private slots:
         statusSpy.clear();
 
         // Start a third sync, this time together with a real file to sync
-        fakeFolder.localModifier().appendByte("C/c1");
+        fakeFolder.localModifier().appendByte(QStringLiteral("C/c1"));
+        fakeFolder.applyLocalModificationsWithoutSync();
         fakeFolder.scheduleSync();
         fakeFolder.execUntilBeforePropagation();
         verifyThatPushMatchesPull(fakeFolder, statusSpy);
@@ -358,8 +379,8 @@ private slots:
         statusSpy.clear();
 
         // Another sync after clearing the blacklist entry, everything should return to order.
-        fakeFolder.syncEngine().journal()->wipeErrorBlacklistEntry("A/a1");
-        fakeFolder.syncEngine().journal()->wipeErrorBlacklistEntry("B/b0");
+        fakeFolder.syncEngine().journal()->wipeErrorBlacklistEntry(QStringLiteral("A/a1"));
+        fakeFolder.syncEngine().journal()->wipeErrorBlacklistEntry(QStringLiteral("B/b0"));
         fakeFolder.scheduleSync();
         fakeFolder.execUntilBeforePropagation();
         verifyThatPushMatchesPull(fakeFolder, statusSpy);
@@ -387,8 +408,9 @@ private slots:
                 {QStringLiteral("a"), 4},
                 {QStringLiteral("a1"), 4}
             }}}}};
-        fakeFolder.serverErrorPaths().append("A/a1");
-        fakeFolder.localModifier().appendByte("A/a1");
+        fakeFolder.serverErrorPaths().append(QStringLiteral("A/a1"));
+        fakeFolder.localModifier().appendByte(QStringLiteral("A/a1"));
+        fakeFolder.applyLocalModificationsWithoutSync();
 
         fakeFolder.scheduleSync();
         fakeFolder.execUntilBeforePropagation();
@@ -412,8 +434,9 @@ private slots:
     // children are marked as OK before their parents do.
     void childOKEmittedBeforeParent() {
         FakeFolder fakeFolder{FileInfo::A12_B12_C12_S12()};
-        fakeFolder.localModifier().appendByte("B/b1");
-        fakeFolder.remoteModifier().appendByte("C/c1");
+        fakeFolder.localModifier().appendByte(QStringLiteral("B/b1"));
+        fakeFolder.applyLocalModificationsWithoutSync();
+        fakeFolder.remoteModifier().appendByte(QStringLiteral("C/c1"));
         StatusPushSpy statusSpy(fakeFolder.syncEngine());
 
         fakeFolder.syncOnce();
@@ -434,9 +457,9 @@ private slots:
         sharedUpToDateStatus.setShared(true);
 
         FakeFolder fakeFolder{FileInfo::A12_B12_C12_S12()};
-        fakeFolder.remoteModifier().insert("S/s0");
-        fakeFolder.remoteModifier().appendByte("S/s1");
-        fakeFolder.remoteModifier().insert("B/b3");
+        fakeFolder.remoteModifier().insert(QStringLiteral("S/s0"));
+        fakeFolder.remoteModifier().appendByte(QStringLiteral("S/s1"));
+        fakeFolder.remoteModifier().insert(QStringLiteral("B/b3"));
         fakeFolder.remoteModifier().find("B/b3")->extraDavProperties = "<oc:share-types><oc:share-type>0</oc:share-type></oc:share-types>";
         fakeFolder.remoteModifier().find("A/a1")->isShared = true; // becomes shared
         fakeFolder.remoteModifier().find("A", true); // change the etags of the parent
@@ -468,9 +491,10 @@ private slots:
 
     void renameError() {
         FakeFolder fakeFolder{FileInfo::A12_B12_C12_S12()};
-        fakeFolder.serverErrorPaths().append("A/a1");
-        fakeFolder.localModifier().rename("A/a1", "A/a1m");
-        fakeFolder.localModifier().rename("B/b1", "B/b1m");
+        fakeFolder.serverErrorPaths().append(QStringLiteral("A/a1"));
+        fakeFolder.localModifier().rename(QStringLiteral("A/a1"), QStringLiteral("A/a1m"));
+        fakeFolder.localModifier().rename(QStringLiteral("B/b1"), QStringLiteral("B/b1m"));
+        fakeFolder.applyLocalModificationsWithoutSync();
         StatusPushSpy statusSpy(fakeFolder.syncEngine());
 
         fakeFolder.scheduleSync();
